/*
* ASIX FORTE DLL library interface
* distributed as-is, without any warranties
* http://www.asix.net/
*
* version 1.1, September 2020
* 
*/

#include "forte.h"

static int KEYS[] = {
    0x10000, 0x10001, 0x10002, 0x10003, 0x10004, 
    0x20000, 0x20001, 
    0x30000, 0x30001, 0x30002, 
    0x40000, 
    0x50000, 0x50001, 0x50002, 0x50003, 0x50004,
    0x60000, 
    0x70000, 
    0x80000, 0x80001, 0x80002,
    0x90000, 0x90001, 
    0xA0000, 
    0xB0000, 0xB0100, 0xB0200, 
    0xC0000, 0xC0100, 0xC0200, 
    0xD0000,     
    0x01, 0x02, 0x04, 0x08,
    (-1) /* terminator */
};

static char* CODES[] = {
    "OPEN_OK", "OPEN_NOTFOUND", "OPEN_CANNOTOPEN", "OPEN_ALREADYOPEN", "OPEN_BADDRIVERVERSION", 
    "CLOSE_OK", "CLOSE_CANNOTCLOSE", 
    "POWERON_OK", "POWERON_OCURR", "POWERON_WRONG_LEVEL",
    "GETPINS_CODE", 
    "OK", "NOT_OPENED", "WRONG_INPUT", "AN_NOT_RUNNING", "ERROR_AN_RUNNING", 
    "SHIFT_BYTE_OUTIN_CODE", 
    "SUPPLY_VOLTAGE_CODE", 
    "VPP_OK", "VPP_OCURR", "VPP_WRONG_LEVEL", 
    "GO_BUTTON_NOT_PRESSED", "GO_BUTTON_PRESSED", 
    "SHIFT_BITS_OUTIN_CODE", 
    "_1WIRE", "_1WIRE_PRESENT", "_1WIRE_NOT_PRESENT", 
    "I2C_CODE", "I2C_ACK", "I2C_NACK",
    "AN_DATA",   
    "FATAL_OVERCURRENTVDD", "FATAL_OVERCURRENTVPP", "FATAL_OVERVOLTAGEVDD", "FATAL_OTHER", 
    "UNKNOWN" /* terminator */
};

HINSTANCE hDLL; /* Handle to DLL */

char* ReturnCodeString(int status)
{
    int i = 0;
    while(KEYS[i] >= 0) { 
        if((KEYS[i] == status) || (KEYS[i]==(status & AN_DATA))) break;       
        i++;
    }
    return CODES[i];
}

int LoadForteDll(char* filename)
{
    int ok = 1;
    hDLL = LoadLibraryA(filename);
    if (hDLL != NULL) {
        AClearFatalError = (LPFNDLLAClearFatalError)GetProcAddress(hDLL, "AClearFatalError");
        AGet = (LPFNDLLAGet)GetProcAddress(hDLL, "AGet");
        AGetBlock = (LPFNDLLAGetBlock)GetProcAddress(hDLL, "AGetBlock");
        AGetBlocking = (LPFNDLLAGetBlocking)GetProcAddress(hDLL, "AGetBlocking");
        AGetProgList = (LPFNDLLAGetProgList)GetProcAddress(hDLL, "AGetProgList");
        AGetStatus = (LPFNDLLAGetStatus)GetProcAddress(hDLL, "AGetStatus");
        Q1WireInit = (LPFNDLLQ1WireInit)GetProcAddress(hDLL, "Q1WireInit");
        Q1WireReadByte = (LPFNDLLQ1WireReadByte)GetProcAddress(hDLL, "Q1WireReadByte");
        Q1WireWriteByte = (LPFNDLLQ1WireWriteByte)GetProcAddress(hDLL, "Q1WireWriteByte");
        QCheckGoButton = (LPFNDLLQCheckGoButton)GetProcAddress(hDLL, "QCheckGoButton");
        QCheckSupplyVoltage = (LPFNDLLQCheckSupplyVoltage)GetProcAddress(hDLL, "QCheckSupplyVoltage");
        QCloseProg = (LPFNDLLQCloseProg)GetProcAddress(hDLL, "QCloseProg");
        QDelay = (LPFNDLLQDelay)GetProcAddress(hDLL, "QDelay");
        QDelay_ns = (LPFNDLLQDelay_ns)GetProcAddress(hDLL, "QDelay_ns");
        QGetPins = (LPFNDLLQGetPins)GetProcAddress(hDLL, "QGetPins");
        QI2CReadByte = (LPFNDLLQI2CReadByte)GetProcAddress(hDLL, "QI2CReadByte");
        QI2CSetSpeed = (LPFNDLLQI2CSetSpeed)GetProcAddress(hDLL, "QI2CSetSpeed");
        QI2CStart = (LPFNDLLQI2CStart)GetProcAddress(hDLL, "QI2CStart");
        QI2CStop = (LPFNDLLQI2CStop)GetProcAddress(hDLL, "QI2CStop");
        QI2CWriteByte = (LPFNDLLQI2CWriteByte)GetProcAddress(hDLL, "QI2CWriteByte");
        QOpenProg = (LPFNDLLQOpenProg)GetProcAddress(hDLL, "QOpenProg");
        QPoweroffVdd = (LPFNDLLQPoweroffVdd)GetProcAddress(hDLL, "QPoweroffVdd");
        QPoweroffVpp = (LPFNDLLQPoweroffVpp)GetProcAddress(hDLL, "QPoweroffVpp");
        QPoweronVdd = (LPFNDLLQPoweronVdd)GetProcAddress(hDLL, "QPoweronVdd");
        QPoweronVpp = (LPFNDLLQPoweronVpp)GetProcAddress(hDLL, "QPoweronVpp");
        QSetActiveLED = (LPFNDLLQSetActiveLED)GetProcAddress(hDLL, "QSetActiveLED");
        QSetGPIOAnswer = (LPFNDLLQSetGPIOAnswer)GetProcAddress(hDLL, "QSetGPIOAnswer");
        QSetPins = (LPFNDLLQSetPins)GetProcAddress(hDLL, "QSetPins");
        QSetPullUpDowns = (LPFNDLLQSetPullUpDowns)GetProcAddress(hDLL, "QSetPullUpDowns");
        QSetShiftSpeed = (LPFNDLLQSetShiftSpeed)GetProcAddress(hDLL, "QSetShiftSpeed");
        QShiftBits = (LPFNDLLQShiftBits)GetProcAddress(hDLL, "QShiftBits");
        QShiftBits_OutIn = (LPFNDLLQShiftBits_OutIn)GetProcAddress(hDLL, "QShiftBits_OutIn");
        QShiftByte = (LPFNDLLQShiftByte)GetProcAddress(hDLL, "QShiftByte");
        QShiftByte_OutIn = (LPFNDLLQShiftByte_OutIn)GetProcAddress(hDLL, "QShiftByte_OutIn");
        QShiftBytes = (LPFNDLLQShiftBytes)GetProcAddress(hDLL, "QShiftBytes");
        QShiftBytes_OutIn = (LPFNDLLQShiftBytes_OutIn)GetProcAddress(hDLL, "QShiftBytes_OutIn");
        QShiftBytes_In = (LPFNDLLQShiftBytes_In)GetProcAddress(hDLL, "QShiftBytes_In");
        QAnI2CStart = (LPFNDLLQAnI2CStart)GetProcAddress(hDLL, "QAnI2CStart");
        QAnStop = (LPFNDLLQAnStop)GetProcAddress(hDLL, "QAnStop");        
                
        if (!AClearFatalError) ok = 0;
        if (!AGet) ok = 0;
        if (!AGetBlock) ok = 0;
        if (!AGetBlocking) ok = 0;
        if (!AGetProgList) ok = 0;
        if (!AGetStatus) ok = 0;
        if (!Q1WireInit) ok = 0;
        if (!Q1WireReadByte) ok = 0;
        if (!Q1WireWriteByte) ok = 0;
        if (!QCheckGoButton) ok = 0;
        if (!QCheckSupplyVoltage) ok = 0;
        if (!QCloseProg) ok = 0;
        if (!QDelay) ok = 0;
        if (!QDelay_ns) ok = 0;
        if (!QGetPins) ok = 0;
        if (!QI2CReadByte) ok = 0;
        if (!QI2CSetSpeed) ok = 0;
        if (!QI2CStart) ok = 0;
        if (!QI2CStop) ok = 0;
        if (!QI2CWriteByte) ok = 0;
        if (!QOpenProg) ok = 0;
        if (!QPoweroffVdd) ok = 0;
        if (!QPoweroffVpp) ok = 0;
        if (!QPoweronVdd) ok = 0;
        if (!QPoweronVpp) ok = 0;
        if (!QSetActiveLED) ok = 0;
        if (!QSetGPIOAnswer) ok = 0;
        if (!QSetPins) ok = 0;
        if (!QSetPullUpDowns) ok = 0;
        if (!QSetShiftSpeed) ok = 0;
        if (!QShiftBits) ok = 0;
        if (!QShiftBits_OutIn) ok = 0;
        if (!QShiftByte) ok = 0;
        if (!QShiftByte_OutIn) ok = 0;
        if (!QShiftBytes) ok = 0;
        if (!QShiftBytes_OutIn) ok = 0;
        if (!QShiftBytes_In) ok = 0;
        if (!QAnI2CStart) ok = 0;
        if (!QAnStop) ok = 0;

        if (ok == 0) {
            FreeLibrary(hDLL);
            return FORTE_DLL_LOAD_FAIL_FUNCTION;
        }
        else {
            return FORTE_DLL_LOAD_OK;
        }
    }
    else {
        return FORTE_DLL_LOAD_FAIL_NAME;
    }
}





