/*
* ASIX FORTE I2C serial EEPROM 24LC02 example
* distributed as-is, without any warranties
* http://www.asix.net/
*
* version 1.0, Dec 2016
* 
* Programmer connection table: 
* 24LC02  <->  FORTE
* A0      <->  GND
* A1      <->  GND
* A2      <->  GND
* VSS     <->  GND
* SDA     <->  D
* SCL     <->  C
* WP      <->  GND
* VCC     <->  VDD
*
*/

#include <stdio.h>
#include "forte.h"

void ReadStatus(char* message)
{
    int status = AGetBlocking();
    char* codestring = ReturnCodeString(status);
    if(message[0] != '\0') {
        printf("%s: %s\n", message, codestring);
    }
    else {
        printf("%s\n", codestring);
    }
}

void PrintHex(int* data, int count)
{
    int i;
    for(i=0;i<count;i++) {
        printf("%02X ", data[i]);
        if(((i+1)%16) == 0) printf("\n");
    }
    if((count%16) != 0) printf("\n");
}

int main(int argc, char *argv[])
{
    int i, count, status, received_bytes;
    int data[256];
    
    printf("ASIX FORTE I2C serial EEPROM 24LC02 example \n");
   
    if(LoadForteDll("forte.dll") == FORTE_DLL_LOAD_OK) {
        printf("DLL load OK\n");
    }
    else {
        printf("DLL load FAILED\n");
        return 1;
    }
    
    QOpenProg(-1);
    ReadStatus("");
    
    // set supply voltage to 3.3V
    QPoweronVdd(20000, 3300);
    ReadStatus("PoweronVdd");
    // 100kHz
    QI2CSetSpeed(I2C_CLK_100kHz);
    ReadStatus("I2CSetSpeed");
    
    // set word address
    QI2CStart(1);
    ReadStatus("I2CStart");
    QI2CWriteByte(0xA0);
    ReadStatus("I2C control byte");
    QI2CWriteByte(0x00);
    ReadStatus("I2C word address");
    // read memory
    count = 256;
    QI2CStart(1);
    ReadStatus("I2CStart");
    QI2CWriteByte(0xA1);
    ReadStatus("I2C control byte");
    for(i=0;i<(count-1);i++) QI2CReadByte(1);
    QI2CReadByte(0);
    QI2CStop();
    // get data
    while(1) {
        status = AGetStatus(&received_bytes);
        if(!status) {
            count = 0;
            break;
        }
        if(received_bytes >= count) {
            AGetBlock(data, count, &received_bytes);
            break;
        }
    }
    // take 8-bit data and print
    for(i=0;i<count;i++) data[i] = data[i] & 0xFF;
    printf("Data:\n");
    PrintHex(data, count);
    
    ReadStatus("I2CStop");

    QCloseProg();
    ReadStatus("");
    
    return 0;
}

