/*
* ASIX FORTE GPIO example
* distributed as-is, without any warranties
* http://www.asix.net/
*
* version 1.0, September 2020
* 
* Sets C->1, D->HiZ, repeatedly reads pins and ends when D is read as 1
* uses programmer supply voltage
*
*/

#include <stdio.h>
#include "forte.h"

void ReadStatus(char* message)
{
    int status = AGetBlocking();
    char* codestring = ReturnCodeString(status);
    if(message[0] != '\0') {
        printf("%s: %s\n", message, codestring);
    }
    else {
        printf("%s\n", codestring);
    }
}

void PrintHex(int* data, int count)
{
    int i;
    for(i=0;i<count;i++) {
        printf("%02X ", data[i]);
        if(((i+1)%16) == 0) printf("\n");
    }
    if((count%16) != 0) printf("\n");
}

unsigned char LsbToMsb(unsigned char databyte)
{
    int i;
    unsigned char x = 0;
    for(i=0;i<8;i++) {
        x = x | (((databyte >> i) & 1) << (7-i));
    }
    return x;
}

int main(int argc, char *argv[])
{
    int done, intdata;
    
    printf("ASIX FORTE GPIO example\n");
    printf("Sets C->1, D->HiZ, repeatedly reads pins and ends when D is read as 1\n");
   
    if(LoadForteDll("forte.dll") == FORTE_DLL_LOAD_OK) {
        printf("DLL load OK\n");
    }
    else {
        printf("DLL load FAILED\n");
        return 1;
    }
    
    QOpenProg(-1);
    ReadStatus("QOpenProg");
    
    
    // set supply voltage to 3.3V
    QPoweronVdd(20000, 3300);
    ReadStatus("PoweronVdd");    
    
    
    // C->1, D->HiZ
    QSetPins((PINS_HIZ<<PINS_D_BIT)|(PINS_HI<<PINS_C_BIT));
    ReadStatus("QSetPins");    
    
    // we suppose that C and D are interconnected
    // wait until the D is read as 1 
    done=0;
    while (done==0){
      QGetPins();
//      intdata=AGetBlocking();
      // -------
      // wait for the value using AGet      
      while (AGet(&intdata)==0);
      // -------
      if ((intdata & GETPINS_CODE)==GETPINS_CODE){
        if ((intdata & GETPINS_PIND) == GETPINS_PIND){
         printf("Pin D = 1, read value %X\n", intdata);
         done=1;        
        }
      }  
      else {
          printf("Wrong answer read, %X\n", intdata);
          done=1;
      }    
    }       
    
    QPoweroffVdd();
    ReadStatus("QPoweroffVdd");
            
    QCloseProg();
    ReadStatus("QCloseProg");
    
    return 0;
}

