/////////////////////////////////////////////////////////////////////////////
// Name:        device.h
// Purpose:     device base class
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

#ifndef _DEVICE_H
#define _DEVICE_H

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/fileconf.h>
#include <wx/mdi.h>
#include <wx/filedlg.h>
#include "wx/wfstream.h"
#include "wx/textfile.h"

#include "childframes.h"
#include "common.h"
#include "string_const.h"
#include "programmer.h"

// ----------------------------------------------------------------------------
// class Device
// ----------------------------------------------------------------------------

class Device;

typedef void (Device::*UserHandler)(void);

#include <wx/dynarray.h>
WX_DECLARE_OBJARRAY(UserHandler, ArrayOfUserHandlers);

class Device: public wxObject
{
public:
    // Constructor and destructor
    Device();
    // variables
    wxMDIParentFrame *parent_window;
    wxFileConfig *config;
    wxMenu *menuAction;
    wxMenu *menuWindow;
    Programmer *prog;
    // functions assumed to be overriden
    virtual wxString GetName();
    virtual wxString GetShortHint();
    virtual wxString GetLongHint();
    virtual int GetMemoriesCount();
    virtual int GetMemoryWordSize(int memory_index); // in bytes
    virtual int GetMemoryWordCount(int memory_index);
    virtual int GetMemoryOffset(int memory_index); // for intel-hex file loading/saving
    virtual wxString GetMemoryName(int memory_index);
    virtual void RegisterActions();
    virtual void RegisterBitAliases();
    virtual void RegisterSettings();
    virtual void NewFile();
    virtual void OpenFile();
    virtual void SaveFileAs();
    // 
    void AddAction(wxString button_label, wxString menu_label, UserHandler user_handler);
    int GetActionCount();
    void Action(int id);
    void SetActionLayout(int column_count);
    void AddBitAlias(wxString label, int memory_index, int word, int bit_start, int bit_count);
    int GetBitAliasCount();
    void BitAliasWindowToScreenBuffer(int index);
    void ScreenBufferToBitAliasWindow();
    void MemoryWindowsToScreenBuffer();
    void ScreenBufferToMemoryWindows();
    void SetScreenBuffer(int memory_index, int word, unsigned char* data);
    void GetScreenBuffer(int memory_index, int word, unsigned char* data);
    void SetScreenBufferBit(int memory_index, int word, int bit, int bit_value);
    int GetScreenBufferBit(int memory_index, int word, int bit);
    void FillScreenBuffer(unsigned char value);
    void SetHiddenBuffer(int memory_index, int word, unsigned char* data);
    void GetHiddenBuffer(int memory_index, int word, unsigned char* data);
    void HiddenBufferToScreenBuffer();
    void ScreenBufferToHiddenBuffer();
    void RefreshScreen();
    void MemoryWindowsToBitAliasWindow();
    void BitAliasWindowToMemoryWindows(wxPGProperty *p);
    void AddSetting(wxPGProperty *prop);

    // window handling
    void Open(wxMDIParentFrame *parent, wxFileConfig *config, wxMenu *menuAction, wxMenu *menuWindow, Programmer *programmer);
    void CreateActionWindow();
    void CreateBitAliasWindow();
    void CreateDeviceHelpWindow();
    void CreateLogWindow();
    void CreateSettingsWindow();
    void CreateMemoryWindows();
    void ConnectWindows();
    void Close();
    void DestroyActionWindow();
    void DestroyBitAliasWindow();
    void DestroyDeviceHelpWindow();
    void DestroyLogWindow();
    void DestroySettingsWindow();
    void DestroyMemoryWindows();
    void ShowAll();
    void HideAll();
    void ShowActionWindow(bool show);
    void ShowBitAliasWindow(bool show);
    void ShowDeviceHelpWindow(bool show);
    void ShowLogWindow(bool show);
    void ShowSettingsWindow(bool show);
    void ShowMemoryWindow(int memory_index, bool show);

    void AddLog(wxString line);
    void AddLog(wxString message, int value);
    void ClearLog();
    void SaveLog(wxString filename);
    void SaveAsBinary(wxString filename);
    void SaveAsHex(wxString filename);
    void OpenAsBinary(wxString filename);
    void OpenAsHex(wxString filename);
    
    bool OpenProg();
    bool CloseProg();
    int ReadStatus(wxString message);

private:
    // variables
    LogWindow* logwindow;
    DeviceHelpWindow* devicehelpwindow;
    SettingsWindow* settingswindow;
    BitAliasWindow* bitaliaswindow;
    ActionWindow* actionwindow;
    MemoryWindow** memorywindows;
    wxArrayPtrVoid settingswindow_props;
    wxArrayPtrVoid bitaliaswindow_aliases;
    wxArrayInt bitaliaswindow_mem;
    wxArrayInt bitaliaswindow_word;
    wxArrayInt bitaliaswindow_bit;
    wxArrayInt bitaliaswindow_len;
    wxArrayString actionwindow_btnlabel;
    wxArrayString actionwindow_mnulabel;
    ArrayOfUserHandlers actionwindow_funct;
    int action_columns;
    unsigned char* screen_buffer;
    unsigned char* hidden_buffer;
    int max_bytes_per_word;
    int max_words_count;
    //
    int ToHex(wxUniChar hex);
    void ToStr(unsigned char data, unsigned char* buf);
};

#endif /* _DEVICE_H*/
