/////////////////////////////////////////////////////////////////////////////
// Name:        programmer.cpp
// Purpose:     programmer interface
// Package:     ASIX FORTE programmer demo software
//              distributed as-is, without any warranties
// Author:      http://www.asix.net/
// Created:     2016
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

// for all others, include the necessary headers (this file is usually all you
// need because it includes almost all "standard" wxWidgets headers)
#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

// standard header file
#include "programmer.h"

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// class Programmer
// ----------------------------------------------------------------------------

int Programmer::KEYS[] = {
    0x10000, 0x10001, 0x10002, 0x10003, 0x10004, 
    0x20000, 0x20001, 
    0x30000, 0x30001, 0x30002, 
    0x40000, 
    0x50000, 0x50001, 0x50002, 
    0x60000, 
    0x70000, 
    0x80000, 0x80001, 0x80002,
    0x90000, 0x90001, 
    0xA0000, 
    0xB0000, 0xB0100, 0xB0200, 
    0xC0000, 0xC0100, 0xC0200, 
    0x01, 0x02, 0x04, 0x08,
    (-1) /* terminator */
};

char* Programmer::CODES[] = {
    "OPEN_OK", "OPEN_NOTFOUND", "OPEN_CANNOTOPEN", "OPEN_ALREADYOPEN", "OPEN_BADDRIVERVERSION", 
    "CLOSE_OK", "CLOSE_CANNOTCLOSE", 
    "POWERON_OK", "POWERON_OCURR", "POWERON_WRONG_LEVEL",
    "GETPINS_CODE", 
    "OK", "NOT_OPENED", "WRONG_INPUT", 
    "SHIFT_BYTE_OUTIN_CODE", 
    "SUPPLY_VOLTAGE_CODE", 
    "VPP_OK", "VPP_OCURR", "VPP_WRONG_LEVEL", 
    "GO_BUTTON_NOT_PRESSED", "GO_BUTTON_PRESSED", 
    "SHIFT_BITS_OUTIN_CODE", 
    "_1WIRE", "_1WIRE_PRESENT", "_1WIRE_NOT_PRESENT", 
    "I2C_CODE", "I2C_ACK", "I2C_NACK", 
    "FATAL_OVERCURRENTVDD", "FATAL_OVERCURRENTVPP", "FATAL_OVERVOLTAGEVDD", "FATAL_OTHER", 
    "UNKNOWN" /* terminator */
};

// constructor
Programmer::Programmer()
{
    preferred_prog_sn = -1;
}

wxString Programmer::ReturnCodeString(int status)
{
    int i = 0;
    while(KEYS[i] >= 0) {
        if(KEYS[i] == status) break;
        i++;
    }
    return wxString(CODES[i]);
}

int Programmer::LoadDll()
{
    int ok = 1;
    // file
    fortedll.Load(STR_DLL_NAME, wxDL_DEFAULT | wxDL_QUIET);
    if(!fortedll.IsLoaded()) return FORTE_DLL_LOAD_FAIL_NAME;
    // functions
    AClearFatalError = (LPFNDLLAClearFatalError)fortedll.GetSymbol("AClearFatalError");
    AGet = (LPFNDLLAGet)fortedll.GetSymbol("AGet");
    AGetBlock = (LPFNDLLAGetBlock)fortedll.GetSymbol("AGetBlock");
    AGetBlocking = (LPFNDLLAGetBlocking)fortedll.GetSymbol("AGetBlocking");
    AGetProgList = (LPFNDLLAGetProgList)fortedll.GetSymbol("AGetProgList");
    AGetStatus = (LPFNDLLAGetStatus)fortedll.GetSymbol("AGetStatus");
    Q1WireInit = (LPFNDLLQ1WireInit)fortedll.GetSymbol("Q1WireInit");
    Q1WireReadByte = (LPFNDLLQ1WireReadByte)fortedll.GetSymbol("Q1WireReadByte");
    Q1WireWriteByte = (LPFNDLLQ1WireWriteByte)fortedll.GetSymbol("Q1WireWriteByte");
    QCheckGoButton = (LPFNDLLQCheckGoButton)fortedll.GetSymbol("QCheckGoButton");
    QCheckSupplyVoltage = (LPFNDLLQCheckSupplyVoltage)fortedll.GetSymbol("QCheckSupplyVoltage");
    QCloseProg = (LPFNDLLQCloseProg)fortedll.GetSymbol("QCloseProg");
    QDelay = (LPFNDLLQDelay)fortedll.GetSymbol("QDelay");
    QDelay_ns = (LPFNDLLQDelay_ns)fortedll.GetSymbol("QDelay_ns");
    QGetPins = (LPFNDLLQGetPins)fortedll.GetSymbol("QGetPins");
    QI2CReadByte = (LPFNDLLQI2CReadByte)fortedll.GetSymbol("QI2CReadByte");
    QI2CSetSpeed = (LPFNDLLQI2CSetSpeed)fortedll.GetSymbol("QI2CSetSpeed");
    QI2CStart = (LPFNDLLQI2CStart)fortedll.GetSymbol("QI2CStart");
    QI2CStop = (LPFNDLLQI2CStop)fortedll.GetSymbol("QI2CStop");
    QI2CWriteByte = (LPFNDLLQI2CWriteByte)fortedll.GetSymbol("QI2CWriteByte");
    QOpenProg = (LPFNDLLQOpenProg)fortedll.GetSymbol("QOpenProg");
    QPoweroffVdd = (LPFNDLLQPoweroffVdd)fortedll.GetSymbol("QPoweroffVdd");
    QPoweroffVpp = (LPFNDLLQPoweroffVpp)fortedll.GetSymbol("QPoweroffVpp");
    QPoweronVdd = (LPFNDLLQPoweronVdd)fortedll.GetSymbol("QPoweronVdd");
    QPoweronVpp = (LPFNDLLQPoweronVpp)fortedll.GetSymbol("QPoweronVpp");
    QSetActiveLED = (LPFNDLLQSetActiveLED)fortedll.GetSymbol("QSetActiveLED");
    QSetGPIOAnswer = (LPFNDLLQSetGPIOAnswer)fortedll.GetSymbol("QSetGPIOAnswer");
    QSetPins = (LPFNDLLQSetPins)fortedll.GetSymbol("QSetPins");
    QSetPullUpDowns = (LPFNDLLQSetPullUpDowns)fortedll.GetSymbol("QSetPullUpDowns");
    QSetShiftSpeed = (LPFNDLLQSetShiftSpeed)fortedll.GetSymbol("QSetShiftSpeed");
    QShiftBits = (LPFNDLLQShiftBits)fortedll.GetSymbol("QShiftBits");
    QShiftBits_OutIn = (LPFNDLLQShiftBits_OutIn)fortedll.GetSymbol("QShiftBits_OutIn");
    QShiftByte = (LPFNDLLQShiftByte)fortedll.GetSymbol("QShiftByte");
    QShiftByte_OutIn = (LPFNDLLQShiftByte_OutIn)fortedll.GetSymbol("QShiftByte_OutIn");
    QShiftBytes = (LPFNDLLQShiftBytes)fortedll.GetSymbol("QShiftBytes");
    QShiftBytes_OutIn = (LPFNDLLQShiftBytes_OutIn)fortedll.GetSymbol("QShiftBytes_OutIn");
    QShiftBytes_In = (LPFNDLLQShiftBytes_In)fortedll.GetSymbol("QShiftBytes_In");
    // check
    if (!AClearFatalError) ok = 0;
    if (!AGet) ok = 0;
    if (!AGetBlock) ok = 0;
    if (!AGetBlocking) ok = 0;
    if (!AGetProgList) ok = 0;
    if (!AGetStatus) ok = 0;
    if (!Q1WireInit) ok = 0;
    if (!Q1WireReadByte) ok = 0;
    if (!Q1WireWriteByte) ok = 0;
    if (!QCheckGoButton) ok = 0;
    if (!QCheckSupplyVoltage) ok = 0;
    if (!QCloseProg) ok = 0;
    if (!QDelay) ok = 0;
    if (!QDelay_ns) ok = 0;
    if (!QGetPins) ok = 0;
    if (!QI2CReadByte) ok = 0;
    if (!QI2CSetSpeed) ok = 0;
    if (!QI2CStart) ok = 0;
    if (!QI2CStop) ok = 0;
    if (!QI2CWriteByte) ok = 0;
    if (!QOpenProg) ok = 0;
    if (!QPoweroffVdd) ok = 0;
    if (!QPoweroffVpp) ok = 0;
    if (!QPoweronVdd) ok = 0;
    if (!QPoweronVpp) ok = 0;
    if (!QSetActiveLED) ok = 0;
    if (!QSetGPIOAnswer) ok = 0;
    if (!QSetPins) ok = 0;
    if (!QSetPullUpDowns) ok = 0;
    if (!QSetShiftSpeed) ok = 0;
    if (!QShiftBits) ok = 0;
    if (!QShiftBits_OutIn) ok = 0;
    if (!QShiftByte) ok = 0;
    if (!QShiftByte_OutIn) ok = 0;
    if (!QShiftBytes) ok = 0;
    if (!QShiftBytes_OutIn) ok = 0;
    if (!QShiftBytes_In) ok = 0;
    //
    if (ok == 0) {
        return FORTE_DLL_LOAD_FAIL_FUNCTION;
    }
    else {
        return FORTE_DLL_LOAD_OK;
    }
}

