#
# ASIX FORTE serial EEPROM 93LC46B example
# distributed as-is, without any warranties
# http://www.asix.net/
#
# version 1.0, Dec 2016
#
# Programmer connection table: 
# 93LC46B <->  FORTE
# CS      <->  P
# VCC     <->  VDD
# VSS     <->  GND
# DI      <->  D
# CLK     <->  C
# DO      <->  I
#

from forte import *

def ReadStatus(message = ""):
    status = AGetBlocking()
    if message != "":
        print(message + ": " + RETURNCODE[status])
    else:
        print(RETURNCODE[status])
        
def PrintHex(data):
    for j in range(len(data)/16):
        s = ""
        for i in range(16):
            s += "%02X "%(data[j*16+i])
        print(s)
    j = len(data)/16
    s = ""
    for i in range(len(data)%16):
        s += "%02X "%(data[j*16+i])
    if s != "":
        print(s)

def LsbToMsb(byte):
    x = 0
    for i in range(8):
        x = x | (((byte >> i) & 1) << (7-i))
    return x;

def main():
    LoadForteDll()
    QOpenProg()
    ReadStatus()
    
    # initiate
    QSetGPIOAnswer(False)
    ReadStatus("SetGPIOAnswer")
    QPoweronVdd(20000, 3300) # supply voltage 3.3V
    ReadStatus("PoweronVdd")
    QSetShiftSpeed(SHIFT_CLK_1000kHz)
    QSetPins((PINS_LO << PINS_P_BIT) | (PINS_LO << PINS_C_BIT) | (PINS_LO << PINS_D_BIT)) # CS=0, CLK=0, DI=0
    QDelay(1)
    QSetPins((PINS_HI << PINS_P_BIT)) # CS=1
    QDelay(1)
    
    # 9-bit command
    QShiftBits(0x003, SHIFT_MODE0, 9)
    # read memory
    count = 128 # bytes
    QShiftBytes_In(SHIFT_MODE1, SHIFT_OUTIN_PINI, count)
    QSetPins((PINS_LO << PINS_P_BIT)) # CS=0
    # get data
    while True:
        status, received_bytes = AGetStatus()
        if not status:
            data = []
            break
        if received_bytes >= count:
            data = AGetBlock(count)
            break
    
    # take 8-bit data and print
    for i in range(len(data)):
        data[i] = LsbToMsb(data[i] & 0xFF)
    print("Data:")
    PrintHex(data)
    
    QSetGPIOAnswer(True)
    ReadStatus("SetGPIOAnswer")
    QCloseProg()
    ReadStatus()


if __name__ == "__main__":
    main()
